/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

/*
 *  The CFrame class models the 'dialog' on the form painter. Much of its
 *  functionality has been 'borrowed' from FXMDIChild to paint such items
 *  as minimize and maximize buttons, for example. It provides a number
 *  of properties that are used by CDispenserDialogFrame.
 */

#include <fx.h>

#include "CFrame.h"

#define BORDERWIDTH 6 // Child border width

///////////////////////////////////////////////////////////
//

FXDEFMAP(CFrame) CFrameMap[]=
{
	FXMAPFUNC(SEL_PAINT,0,CFrame::onPaint),
};

FXIMPLEMENT(CFrame,FXComposite,CFrameMap,ARRAYNUMBER(CFrameMap))

CFrame::CFrame()
{
	flags|=FLAG_ENABLED|FLAG_SHOWN;
	windowbtn=0;
	minimizebtn=0;
	restorebtn=0;
	maximizebtn=0;
	deletebtn=0;
	font=(FXFont*)-1;
	baseColor=0;
	hiliteColor=0;
	shadowColor=0;
	borderColor=0;
	titleColor=0;
	titleBackColor=0;

	title[0] = '\0';
}


CFrame::CFrame(FXComposite* p,const FXString& name,FXIcon* ic,FXMenuPane* mn,FXuint opts,FXint x,FXint y,FXint w,FXint h):
  FXComposite(p,opts,x,y,w,h)
{
	flags|=FLAG_ENABLED|FLAG_SHOWN;
	windowbtn=0;
	minimizebtn=0;
	restorebtn=0;
	maximizebtn=0;
	deletebtn=new FXMDIDeleteButton(this,this,FXWindow::ID_MDI_CLOSE,FRAME_RAISED);
	baseColor=getApp()->baseColor;
	hiliteColor=getApp()->hiliteColor;
	shadowColor=getApp()->shadowColor;
	borderColor=getApp()->borderColor;
	titleColor=getApp()->selforeColor;
	titleBackColor=getApp()->selbackColor;
	font=getApp()->getNormalFont();

	memcpy(title, name.text(), name.length());
	title[name.length()] = '\0';

	windowbtn=new FXMDIWindowButton(this,this,FXWindow::ID_MDI_WINDOW);
	minimizebtn=new FXMDIMinimizeButton(this,this,FXWindow::ID_MDI_MINIMIZE,FRAME_RAISED);
	restorebtn=new FXMDIRestoreButton(this,this,FXWindow::ID_MDI_RESTORE,FRAME_RAISED);
	maximizebtn=new FXMDIMaximizeButton(this,this,FXWindow::ID_MDI_MAXIMIZE,FRAME_RAISED);
}

// Destruct thrashes the pointers
CFrame::~CFrame()
{
	windowbtn=(FXMenuButton*)-1;
	minimizebtn=(FXButton*)-1;
	restorebtn=(FXButton*)-1;
	maximizebtn=(FXButton*)-1;
	deletebtn=(FXButton*)-1;
	font=(FXFont*)-1;
}

void CFrame::create()
{
	FXComposite::create();
	font->create();
	recalc();
}

void CFrame::detach()
{
	FXComposite::detach();
	font->detach();
}

//
// layout code stolen from FXTopWindow which supports LAYOUT_FIX_xxxxx
//
#define LAYOUT_SIDE_MASK (LAYOUT_SIDE_LEFT|LAYOUT_SIDE_RIGHT|LAYOUT_SIDE_TOP|LAYOUT_SIDE_BOTTOM)
// Layout modes
#define LAYOUT_MASK (LAYOUT_SIDE_MASK|LAYOUT_RIGHT|LAYOUT_CENTER_X|LAYOUT_BOTTOM|LAYOUT_CENTER_Y|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y)
const FXuint hspacing = 0;
const FXuint vspacing = 0;


void CFrame::layout(){
//  FXWindow *contents=contentWindow();
  FXint th,fh,mw,mh,bw,bh,by,bx;
  fh=font->getFontHeight();

	mw=0;
	mh=0;
	if ( windowbtn )
	{
		mw=windowbtn->getDefaultWidth();
		mh=windowbtn->getDefaultHeight();
	}

	bw=0;
	bh=0;
	if ( deletebtn )
	{
		bw=deletebtn->getDefaultWidth();
		bh=deletebtn->getDefaultHeight();
	}

	th=FXMAX3(fh,mh,bh)+4;
	bx=width-BORDERWIDTH-bw-2;
	by=BORDERWIDTH+(th-bh)/2;

	windowbtn->position(BORDERWIDTH+2,BORDERWIDTH+(th-mh)/2,mw,mh);

	FXint ux = bx;
	if ( deletebtn->isEnabled() )
	{
		deletebtn->position(bx,by,bw,bh); bx-=bw+3;
		deletebtn->show();
	}

	if ( windowbtn->isEnabled() )
	{
		if ( maximizebtn->isEnabled() || minimizebtn->isEnabled() )
		{
			maximizebtn->position(bx,by,bw,bh); bx-=bw+3;
			if ( maximizebtn->isEnabled() )
			{
				maximizebtn->show();
			}

			minimizebtn->position(bx,by,bw,bh);
			if ( minimizebtn->isEnabled() )
			{
				minimizebtn->show();
			}
		}
		else
		{
			maximizebtn->position(ux,by,bw,bh);
			minimizebtn->position(ux,by,bw,bh);
		}
	}
	else
	{
		maximizebtn->position(-50,-50,0,0);
		minimizebtn->position(-50,-50,0,0);
		windowbtn->position(-50,-50,0,0);
		deletebtn->position(-50,-50,0,0);
	}

	FXWindow* child;
  
	for(child=getFirst(); child; child=child->getNext())
	{
		if (
			child != deletebtn &&
			child != minimizebtn &&
			child != maximizebtn &&
			child != restorebtn &&
			child != windowbtn
			)
		{
			child->recalc();
			child->position(BORDERWIDTH, 27, width-BORDERWIDTH*2, height-BORDERWIDTH-27);
		}
	}

	flags&=~FLAG_DIRTY;
  }

// Handle repaint 
long CFrame::onPaint(FXObject*,FXSelector,void* ptr){
  FXEvent *ev=(FXEvent*)ptr;
  FXint xx,yy,tw,th,titlespace,letters,dots,dotspace;
  FXint fh,mh,bh,bw,mw;
  FXDCWindow dc(this,ev);
  
  fh=font->getFontHeight();

	mw=0;
	mh=0;

//	if ( windowbtn )
//	{
//		mw=windowbtn->getDefaultWidth();
//		mh=windowbtn->getDefaultHeight();
//	}

//	if ( windowbtn->isEnabled() )
//	{
//		mw=windowbtn->getDefaultWidth();
//		mh=windowbtn->getDefaultHeight();
//	}
//	else
//	{
		mw=windowbtn->getWidth();
		mh=windowbtn->getHeight();
//	}

  bw=deletebtn->getDefaultWidth();
  bh=deletebtn->getDefaultHeight();
  th=FXMAX3(fh,mh,bh)+4;

  dc.setClipRectangle(ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);
  dc.setForeground(baseColor);
  
  // Draw MDIChild background
  dc.fillRectangle(ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);
  
  // Draw border
  dc.setForeground(baseColor);
  dc.drawLine(0,0,width-2,0);
  dc.drawLine(0,0,0,height-2);
  dc.setForeground(hiliteColor);
  dc.drawLine(1,0+1,width-3,1);
  dc.drawLine(1,0+1,1,height-3);
  dc.setForeground(shadowColor);
  dc.drawLine(1,height-2,width-2,height-2);
  dc.drawLine(width-2,height-2,width-2,01);
  dc.setForeground(borderColor);
  dc.drawLine(0,height-1,width-1,height-1);
  dc.drawLine(width-1,0,width-1,height-1);
  
  // Draw title background
//  dc.setForeground(isActive() ? (hasFocus() ? titleBackColor : shadowColor) : backColor);
  dc.setForeground(titleBackColor);
  dc.fillRectangle(BORDERWIDTH,BORDERWIDTH,width-BORDERWIDTH*2,th);
  
  // Draw title
  FXString strTitle = title;
  if(strTitle.text()){
    xx=BORDERWIDTH+mw+1+4;
    yy=BORDERWIDTH+font->getFontAscent()+(th-fh)/2;
    
    // Compute space for title
    titlespace=width-mw-3*bw-(BORDERWIDTH<<1)-1-4-4-6-1;
    
    dots=0;
    letters=strTitle.length();
    
    // Title too large for space
    if(font->getTextWidth(strTitle.text(),letters)>titlespace){
      dotspace=titlespace-font->getTextWidth("...",3);
      while(letters>0 && (tw=font->getTextWidth(strTitle.text(),letters))>dotspace) letters--;
      dots=3;
      if(letters==0){
        letters=1;
        dots=0;
        }
      }
    
    // Draw as much of the title as possible
//    dc.setForeground(isActive() ? titleColor : borderColor);
	dc.setForeground(titleColor);
    dc.setTextFont(font);
    dc.drawText(xx,yy,strTitle.text(),letters);
    dc.drawText(xx+font->getTextWidth(strTitle.text(),letters),yy,"...",dots);
    }
  return 1;
  }

void CFrame::setBaseColor(FXColor clr)
{
	baseColor=clr;
	update();
}

void CFrame::setHiliteColor(FXColor clr)
{
	hiliteColor=clr;
	update();
}

void CFrame::setShadowColor(FXColor clr)
{
	shadowColor=clr;
	update();
}

void CFrame::setBorderColor(FXColor clr)
{
	borderColor=clr;
	update();
}

void CFrame::setTitle(const FXString& name)
{
	if(title!=name)
	{
		memcpy(title, name.text(), name.length());
		title[name.length()] = '\0';
		update();
    }
}

FXString CFrame::getTitle() const
{
//	FXString s = title;
	return title;
}

void CFrame::setFont(FXFont *fnt)
{
	if(!fnt){ fxerror("%s::setFont: NULL font specified.\n",getClassName()); }
	if(font!=fnt)
	{
		font=fnt;
		recalc();
		update();
	}
}

void CFrame::setMinimizeButton(FXbool bShow)
{
	if ( bShow )
		minimizebtn->enable();
	else
		minimizebtn->disable();
}

FXbool CFrame::getMinimizeButton(void)
{
	if ( minimizebtn )
		return minimizebtn->isEnabled();

	return FALSE;
}

void CFrame::setMaximizeButton(FXbool bShow)
{
	if ( bShow )
		maximizebtn->enable();
	else
		maximizebtn->disable();
}

FXbool CFrame::getMaximizeButton(void)
{
	if ( maximizebtn )
		return maximizebtn->isEnabled();

	return FALSE;
}

void CFrame::setMenuButton(FXbool bShow)
{
	if ( bShow )
		windowbtn->enable();
	else
		windowbtn->disable();

	update();
}

FXbool CFrame::getMenuButton(void)
{
	if ( windowbtn )
		return windowbtn->isEnabled();

	return FALSE;
}

