/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserMatrix.h"

#include <CPropertyBackColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyFrame.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T, FXint I>
class CPropertyNumColumns : public CProperty
{
public:
	CPropertyNumColumns(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXint I>
const char* CPropertyNumColumns<T, I>::name(void)
{
	return "Columns";
}

template <class T, FXint I>
ePropertyType CPropertyNumColumns<T, I>::type(void)
{
	return PROPERTY_TEXT;
}

template <class T, FXint I>
const char* CPropertyNumColumns<T, I>::description(void)
{
	return "Sets or returns the number of\ncolumns associated with this object";
}

template <class T, FXint I>
const char* CPropertyNumColumns<T, I>::tooltip(void)
{
	return "getNumColumns()\nsetNumColumns()";
}

template <class T, FXint I>
void CPropertyNumColumns<T, I>::set(FXWindow* pWindow, FXString strValue)
{
	FXMatrix* p = static_cast<FXMatrix*>(pWindow);

	FXint n = getIntFromString(strValue);

	// FXMatrix will explode if we don't do this.
	if ( n < 1 )
		n = 1;

	p->setNumColumns( n );
}

template <class T, FXint I>
FXString CPropertyNumColumns<T, I>::get(FXWindow* pWindow)
{
	FXMatrix* p = static_cast<FXMatrix*>(pWindow);
	return getStringFromInt( p->getNumColumns() );
}

template <class T, FXint I>
FXString CPropertyNumColumns<T, I>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p && p->getNumColumns() != I )
	{
		return "\t" + widget + "->setNumColumns(" + get(pWindow) + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

template <class T, FXint I>
class CPropertyNumRows : public CProperty
{
public:
	CPropertyNumRows(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXint I>
const char* CPropertyNumRows<T, I>::name(void)
{
	return "Rows";
}

template <class T, FXint I>
ePropertyType CPropertyNumRows<T, I>::type(void)
{
	return PROPERTY_TEXT;
}

template <class T, FXint I>
const char* CPropertyNumRows<T, I>::description(void)
{
	return "Sets or returns the number of\nrows associated with this object";
}

template <class T, FXint I>
const char* CPropertyNumRows<T, I>::tooltip(void)
{
	return "getNumRows()\nsetNumRows()";
}

template <class T, FXint I>
void CPropertyNumRows<T, I>::set(FXWindow* pWindow, FXString strValue)
{
	FXMatrix* p = static_cast<FXMatrix*>(pWindow);

	FXint n = getIntFromString(strValue);

	// FXMatrix will explode if we don't do this.
	if ( n < 1 )
		n = 1;

	p->setNumRows( n );
}

template <class T, FXint I>
FXString CPropertyNumRows<T, I>::get(FXWindow* pWindow)
{
	FXMatrix* p = static_cast<FXMatrix*>(pWindow);
	return getStringFromInt( p->getNumRows() );
}

template <class T, FXint I>
FXString CPropertyNumRows<T, I>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p && p->getNumRows() != I )
	{
		return "\t" + widget + "->setNumRows(" + get(pWindow) + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyMatrixStyle : public CProperty
{
public:
	CPropertyMatrixStyle(IDispenser* pDispenser) : CProperty(pDispenser) {}

	virtual const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	virtual void set(FXWindow* pWindow, FXString strValue);
	virtual FXString get(FXWindow* pWindow);
	virtual void validate(IConnector* pConnector, FXString& strNewValue) {}
	virtual void validate(IConnector* pConnector);

	OPTION* option(void);

private:
	static OPTION Style[];
};

// From FXMatrix.h
template <class T>
OPTION CPropertyMatrixStyle<T>::Style[] = 
{
	{"MATRIX_BY_ROWS"    , MATRIX_BY_ROWS     ,"Fixed number of rows, add columns as needed"},
	{"MATRIX_BY_COLUMNS" , MATRIX_BY_COLUMNS  ,"Fixed number of columns, adding rows as needed"},
	{0, 0, 0}
};

template <class T>
const char* CPropertyMatrixStyle<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertyMatrixStyle<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyMatrixStyle<T>::description(void)
{
	return "The matrix style for this object.";
}

template <class T>
const char* CPropertyMatrixStyle<T>::tooltip(void)
{
	return "getMatrixStyle()\nsetMatrixStyle()";
}

template <class T>
void CPropertyMatrixStyle<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setMatrixStyle( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyMatrixStyle<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getMatrixStyle() );
}

template <class T>
void CPropertyMatrixStyle<T>::validate(IConnector* pConnector)
{
	T* p = static_cast<T*>(pConnector->getWidget());
	FXint nValue = p->getMatrixStyle();

	IProperty* pProperty = 0;
	if ( getProperty("Columns", &pProperty) )
	{
		if ( nValue & MATRIX_BY_COLUMNS )
			pProperty->enable();
		else
			pProperty->disable();

		pConnector->refreshProperty();
	}

	pProperty = 0;
	if ( getProperty("Rows", &pProperty) )
	{
		if ( nValue & MATRIX_BY_COLUMNS )
			pProperty->disable();
		else
			pProperty->enable();

		pConnector->refreshProperty();
	}
}

template <class T>
OPTION* CPropertyMatrixStyle<T>::option(void)
{
	return Style;
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char Matrix[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x08,0x08,0x08,0x84,0x84,0x84,0xc6,0xc6,0xc6,0xce,0xce,0xce,0xd6,0xad,0x00,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x50,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0x3a,0xac,0x6c,0xeb,0xbe,0x70,
  0x2c,0xcb,0xda,0x6c,0xdf,0x78,0x0c,0x64,0x79,0xef,0xd3,0xbb,0xcb,0x6f,0x48,0xdc,
  0x05,0x2b,0xc4,0x64,0x4f,0x70,0x44,0xb6,0x4c,0x86,0x16,0x60,0x35,0xfd,0x19,0x85,
  0x4f,0x80,0x76,0xcb,0xed,0x76,0x07,0x86,0xc9,0xa0,0x3a,0xdd,0x44,0xb3,0x18,0x17,
  0xd3,0xc0,0x6e,0xbb,0xdf,0x6e,0xb0,0x80,0x3d,0x9e,0x4e,0xe1,0xee,0xb9,0xeb,0x6a,
  0x51,0x03,0xf0,0x80,0x71,0x61,0x67,0x4b,0x84,0x2b,0x4c,0x69,0x59,0x81,0x8b,0x72,
  0x86,0x38,0x7a,0x52,0x88,0x7d,0x8a,0x8b,0x80,0x8d,0x3f,0x90,0x2c,0x92,0x93,0x9a,
  0x7f,0x95,0x78,0x97,0x3e,0x99,0x54,0x9b,0x14,0x7e,0x9f,0xa0,0x83,0x98,0x8e,0xa5,
  0xa6,0x94,0xa8,0x82,0xa3,0x8f,0x8e,0x7c,0x4e,0x9d,0xb0,0x6f,0xa1,0x85,0x7e,0x89,
  0xb7,0xb8,0x6d,0xba,0x39,0xb2,0xb5,0xae,0xbe,0xbf,0x51,0xaa,0xa2,0xac,0x4d,0xc5,
  0x87,0x9e,0xc7,0xc1,0xb3,0x7b,0xad,0x12,0xa7,0xc7,0xc8,0xb2,0x37,0xb2,0xd4,0xd5,
  0xaf,0xbf,0xd1,0xda,0xb4,0xdc,0xd6,0xd0,0xc9,0xbb,0x68,0x58,0xc6,0xdf,0xe6,0xc2,
  0xe2,0xcc,0xdd,0xea,0xb8,0xe0,0x36,0xdb,0xef,0xf0,0xce,0xd7,0xd8,0x8e,0xe1,0xd3,
  0xf6,0xe4,0xeb,0xd9,0xe8,0x2d,0xeb,0x75,0x48,0x49,0x98,0x11,0xee,0xd2,0xad,0xc8,
  0xc7,0x30,0x17,0x3a,0x4e,0x0b,0x1b,0x56,0x51,0xc2,0x82,0x98,0x98,0x16,0x67,0x08,
  0x68,0xdc,0x48,0x40,0x0b,0xc7,0x8d,0xfe,0xce,0x92,0x08,0xe0,0xa5,0x10,0xcc,0x8a,
  0x8f,0x1d,0xb7,0xa0,0x0c,0x49,0xb1,0xe2,0x38,0x8c,0x27,0x39,0x1a,0x49,0xf9,0x91,
  0xe5,0x90,0x91,0x0f,0x6d,0x2d,0x8c,0xb9,0x11,0x40,0x00,0x2d,0x06,0x00,0xd4,0x1c,
  0xc0,0xe4,0x89,0x19,0x19,0x16,0xef,0x99,0x1c,0xf0,0x11,0x40,0x81,0xa7,0x1d,0x87,
  0x16,0xed,0xe4,0x45,0x25,0x21,0x9c,0x9d,0x08,0x2e,0x6d,0xfa,0xb4,0x40,0x54,0x8e,
  0x36,0x89,0x3e,0x83,0x43,0x60,0x9f,0x4b,0x7f,0x30,0x99,0xca,0x7c,0xca,0xc6,0x23,
  0x48,0xb1,0xde,0xda,0xfc,0x29,0xab,0x89,0x24,0xc4,0xad,0x3d,0xe7,0xb8,0x11,0xfa,
  0x76,0x2a,0xbe,0x37,0x40,0xe9,0xea,0x78,0xc9,0x22,0x63,0x4f,0xb2,0x7c,0x09,0x84,
  0x5d,0xf3,0x6c,0x8b,0x01,0xc1,0x44,0xed,0xea,0xc4,0xdb,0xb1,0x4d,0x59,0xb9,0x7c,
  0x17,0xff,0x01,0xea,0xf8,0xb1,0x59,0x52,0x68,0x0b,0xf3,0xf4,0x24,0xe0,0x72,0xdb,
  0xcc,0x70,0x6f,0x75,0xb1,0x7c,0x55,0x72,0x33,0xbc,0x3b,0xd8,0xe8,0xc5,0x2c,0x54,
  0xf3,0xe9,0xb1,0x90,0x5f,0x24,0xc5,0xba,0x53,0x6d,0x01,0xbd,0xb3,0x6f,0x2b,0x4e,
  0xfd,0x37,0xe8,0x1b,0xc8,0xbc,0xc5,0x6a,0xcd,0xd8,0x15,0x4f,0x01,0x2d,0x5e,0x6d,
  0x03,0xca,0xdd,0xaf,0xe4,0x99,0xae,0x6c,0xdd,0x10,0xc0,0x5e,0x60,0x31,0xca,0x8f,
  0xad,0x73,0xbe,0xbe,0x8e,0x1d,0x30,0xf7,0xb0,0xa8,0x90,0x12,0xee,0xbd,0xbd,0x7c,
  0x50,0xe8,0x5d,0x87,0xfb,0xc5,0xe4,0xfa,0xa2,0x68,0xdf,0x4f,0x99,0x70,0x69,0xaf,
  0x31,0x2c,0xc5,0xdd,0x2e,0x7c,0x58,0xa6,0xcd,0x44,0xf4,0x89,0x67,0xdf,0x42,0x25,
  0x10,0xf8,0xdf,0x7a,0x75,0x54,0xe5,0xa0,0x17,0x23,0x45,0x18,0x99,0x84,0x14,0x4e,
  0x58,0x9f,0x52,0x2d,0x65,0x08,0x5a,0x49,0x1a,0x66,0xc8,0x4d,0x72,0x1d,0xb6,0x04,
  0x60,0x88,0x1d,0x7e,0x48,0x62,0x87,0xbb,0x21,0x72,0xc5,0x8a,0x2a,0xb6,0xc8,0xe2,
  0x8b,0x2e,0xc6,0x08,0xa3,0x3d,0x13,0xcc,0x28,0xe3,0x8d,0x36,0xe6,0x78,0x63,0x06,
  0x3a,0xf6,0x88,0xe3,0x8f,0x30,0xa6,0x20,0xe4,0x90,0x44,0xa2,0x10,0x01,0x00,0x00,
  0x3b
  };

CDispenserMatrix::CDispenserMatrix()
{
	m_Properties.append(new CPropertyLayout<FXMatrix>(this));
	m_Properties.append(new CPropertyFrame<FXMatrix>(this));
	m_Properties.append(new CPropertyMatrixStyle<FXMatrix>(this));
	m_Properties.append(new CPropertyBackColor<FXMatrix, FXRGB(255, 255, 255)>(this));
	m_Properties.append(new CPropertyNumRows<FXMatrix, 0>(this));
	m_Properties.append(new CPropertyNumColumns<FXMatrix, 0>(this));
	m_Properties.append(new CPropertyPadTop<FXMatrix>(this));
	m_Properties.append(new CPropertyPadLeft<FXMatrix>(this));
	m_Properties.append(new CPropertyPadBottom<FXMatrix>(this));
	m_Properties.append(new CPropertyPadRight<FXMatrix>(this));
	m_Properties.append(new CPropertyLeft<FXMatrix>(this));
	m_Properties.append(new CPropertyTop<FXMatrix>(this));
	m_Properties.append(new CPropertyWidth<FXMatrix>(this));
	m_Properties.append(new CPropertyHeight<FXMatrix>(this));
}

const char* CDispenserMatrix::getWidgetName(void)
{
	return "FXMatrix";
}

const void* CDispenserMatrix::getWidgetIcon(void)
{
	return Matrix;
}

const char* CDispenserMatrix::getWidgetCategory(void)
{
	return "Containers";
}

const int CDispenserMatrix::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserMatrix::getDefaultWidth(void)
{
	return 120;
}

FXint CDispenserMatrix::getDefaultHeight(void)
{
	return 120;
}

FXint CDispenserMatrix::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserMatrix::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserMatrix::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserMatrix::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserMatrix::getClassInfo(void)
{
	return &FXMatrix::metaClass;
}

FXbool CDispenserMatrix::allowChild(const FXMetaClass*)
{
	return TRUE;
}

FXbool CDispenserMatrix::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserMatrix::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserMatrix::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXMatrix::metaClass);
}

IDispenser* CDispenserMatrix::clone(void)
{
	return new CDispenserMatrix;
}

FXWindow* CDispenserMatrix::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXMatrix(p, 1, 
				LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserMatrix::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXMatrix(" << parent << ", 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << ", " << getProperty("Top").get(pWindow) << ", ";
	stream << getProperty("Width").get(pWindow) << ", " << getProperty("Height").get(pWindow);

	stream  << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}

