/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserSlider.h"

#include <CPropertyBackColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyFrame.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyTipText.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>
#include <CPropertyRange.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertySliderStyle : public CProperty
{
public:
	CPropertySliderStyle(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	virtual void set(FXWindow* pWindow, FXString strValue);
	virtual FXString get(FXWindow* pWindow);
	virtual void validate(IConnector* pConnector, FXString& strNewValue);
	virtual void validate(IConnector* pConnector) {}

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

///////////////////////////////////////////////////////////////////////////////
//

// From FXSlider.h
template <class T>
OPTION CPropertySliderStyle<T>::Style[] = 
{
	{"SLIDER_HORIZONTAL" , SLIDER_HORIZONTAL , "Slider shown horizontally"},
	{"SLIDER_VERTICAL"   , SLIDER_VERTICAL   , "Slider shown vertically"},
	{"SLIDER_ARROW_UP"   , SLIDER_ARROW_UP   , "Slider has arrow head pointing up"},
	{"SLIDER_ARROW_DOWN" , SLIDER_ARROW_DOWN , "Slider has arrow head pointing down"},
	{"SLIDER_ARROW_LEFT" , SLIDER_ARROW_LEFT , "Slider has arrow head pointing left"},
	{"SLIDER_ARROW_RIGHT", SLIDER_ARROW_RIGHT, "Slider has arrow head pointing right"},
	{"SLIDER_INSIDE_BAR" , SLIDER_INSIDE_BAR , "Slider is inside the slot rather than overhanging"},
	{0, 0, 0}
};

template <class T>
const char* CPropertySliderStyle<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertySliderStyle<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertySliderStyle<T>::description(void)
{
	return "The slider style of this object";
}

template <class T>
const char* CPropertySliderStyle<T>::tooltip(void)
{
	return "getSliderStyle()\nsetSliderStyle()";
}

template <class T>
void CPropertySliderStyle<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setSliderStyle( getIntFromString(strValue) );
}

template <class T>
FXString CPropertySliderStyle<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getSliderStyle() );
}

template <class T>
void CPropertySliderStyle<T>::validate(IConnector* pConnector, FXString& strNewValue)
{
	T* p = static_cast<T*>(pConnector->getWidget());
	FXint nOldValue = p->getSliderStyle();
	FXint nNewValue = getIntFromString(strNewValue);

	toggleAttribute(nOldValue, nNewValue, SLIDER_ARROW_UP, SLIDER_ARROW_DOWN);

	strNewValue = getStringFromInt(nNewValue);
}

template <class T>
OPTION* CPropertySliderStyle<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertySliderStyle<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p->getSliderStyle() == 0 )
		return;

	CProperty::getOptions(pWindow, strOptions);
}

///////////////////////////////////////////////////////////////////////////////
//

template <class T, FXColor C>
class CPropertySlotColor : public CProperty
{
public:
	CPropertySlotColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXColor C>
const char* CPropertySlotColor<T, C>::name(void)
{
	return "SlotColor";
}

template <class T, FXColor C>
ePropertyType CPropertySlotColor<T, C>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T, FXColor C>
const char* CPropertySlotColor<T, C>::description(void)
{
	return "The slot color of this object";
}

template <class T, FXColor C>
const char* CPropertySlotColor<T, C>::tooltip(void)
{
	return "getSlotColor()\nsetSlotColor()";
}

template <class T, FXColor C>
void CPropertySlotColor<T, C>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setSlotColor( getColorFromString(strValue) );
}

template <class T, FXColor C>
FXString CPropertySlotColor<T, C>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getSlotColor() );
}

template <class T, FXColor C>
FXString CPropertySlotColor<T, C>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);

	if ( p->getSlotColor() != C )
	{
		FXString str = getFXRGBString(p->getSlotColor());
		return "\t" + widget + "->setSlotColor(" + str + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char Slider[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xf7,0x00,0x00,0x00,0x00,0x00,
  0x80,0x00,0x00,0x00,0x80,0x00,0x80,0x80,0x00,0x00,0x00,0x80,0x80,0x00,0x80,0x00,
  0x80,0x80,0xc0,0xc0,0xc0,0xc0,0xdc,0xc0,0xa6,0xca,0xf0,0x00,0x00,0x00,0x00,0x00,
  0x2a,0x00,0x00,0x55,0x00,0x00,0x7f,0x00,0x00,0xaa,0x00,0x00,0xd4,0x00,0x2a,0x00,
  0x00,0x2a,0x2a,0x00,0x2a,0x55,0x00,0x2a,0x7f,0x00,0x2a,0xaa,0x00,0x2a,0xd4,0x00,
  0x55,0x00,0x00,0x55,0x2a,0x00,0x55,0x55,0x00,0x55,0x7f,0x00,0x55,0xaa,0x00,0x55,
  0xd4,0x00,0x7f,0x00,0x00,0x7f,0x2a,0x00,0x7f,0x55,0x00,0x7f,0x7f,0x00,0x7f,0xaa,
  0x00,0x7f,0xd4,0x00,0xaa,0x00,0x00,0xaa,0x2a,0x00,0xaa,0x55,0x00,0xaa,0x7f,0x00,
  0xaa,0xaa,0x00,0xaa,0xd4,0x00,0xd4,0x00,0x00,0xd4,0x2a,0x00,0xd4,0x55,0x00,0xd4,
  0x7f,0x00,0xd4,0xaa,0x00,0xd4,0xd4,0x2a,0x00,0x00,0x2a,0x00,0x2a,0x2a,0x00,0x55,
  0x2a,0x00,0x7f,0x2a,0x00,0xaa,0x2a,0x00,0xd4,0x2a,0x2a,0x00,0x2a,0x2a,0x2a,0x2a,
  0x2a,0x55,0x2a,0x2a,0x7f,0x2a,0x2a,0xaa,0x2a,0x2a,0xd4,0x2a,0x55,0x00,0x2a,0x55,
  0x2a,0x2a,0x55,0x55,0x2a,0x55,0x7f,0x2a,0x55,0xaa,0x2a,0x55,0xd4,0x2a,0x7f,0x00,
  0x2a,0x7f,0x2a,0x2a,0x7f,0x55,0x2a,0x7f,0x7f,0x2a,0x7f,0xaa,0x2a,0x7f,0xd4,0x2a,
  0xaa,0x00,0x2a,0xaa,0x2a,0x2a,0xaa,0x55,0x2a,0xaa,0x7f,0x2a,0xaa,0xaa,0x2a,0xaa,
  0xd4,0x2a,0xd4,0x00,0x2a,0xd4,0x2a,0x2a,0xd4,0x55,0x2a,0xd4,0x7f,0x2a,0xd4,0xaa,
  0x2a,0xd4,0xd4,0x55,0x00,0x00,0x55,0x00,0x2a,0x55,0x00,0x55,0x55,0x00,0x7f,0x55,
  0x00,0xaa,0x55,0x00,0xd4,0x55,0x2a,0x00,0x55,0x2a,0x2a,0x55,0x2a,0x55,0x55,0x2a,
  0x7f,0x55,0x2a,0xaa,0x55,0x2a,0xd4,0x55,0x55,0x00,0x55,0x55,0x2a,0x55,0x55,0x55,
  0x55,0x55,0x7f,0x55,0x55,0xaa,0x55,0x55,0xd4,0x55,0x7f,0x00,0x55,0x7f,0x2a,0x55,
  0x7f,0x55,0x55,0x7f,0x7f,0x55,0x7f,0xaa,0x55,0x7f,0xd4,0x55,0xaa,0x00,0x55,0xaa,
  0x2a,0x55,0xaa,0x55,0x55,0xaa,0x7f,0x55,0xaa,0xaa,0x55,0xaa,0xd4,0x55,0xd4,0x00,
  0x55,0xd4,0x2a,0x55,0xd4,0x55,0x55,0xd4,0x7f,0x55,0xd4,0xaa,0x55,0xd4,0xd4,0x7f,
  0x00,0x00,0x7f,0x00,0x2a,0x7f,0x00,0x55,0x7f,0x00,0x7f,0x7f,0x00,0xaa,0x7f,0x00,
  0xd4,0x7f,0x2a,0x00,0x7f,0x2a,0x2a,0x7f,0x2a,0x55,0x7f,0x2a,0x7f,0x7f,0x2a,0xaa,
  0x7f,0x2a,0xd4,0x7f,0x55,0x00,0x7f,0x55,0x2a,0x7f,0x55,0x55,0x7f,0x55,0x7f,0x7f,
  0x55,0xaa,0x7f,0x55,0xd4,0x7f,0x7f,0x00,0x7f,0x7f,0x2a,0x7f,0x7f,0x55,0x7f,0x7f,
  0x7f,0x7f,0x7f,0xaa,0x7f,0x7f,0xd4,0x7f,0xaa,0x00,0x7f,0xaa,0x2a,0x7f,0xaa,0x55,
  0x7f,0xaa,0x7f,0x7f,0xaa,0xaa,0x7f,0xaa,0xd4,0x7f,0xd4,0x00,0x7f,0xd4,0x2a,0x7f,
  0xd4,0x55,0x7f,0xd4,0x7f,0x7f,0xd4,0xaa,0x7f,0xd4,0xd4,0xaa,0x00,0x00,0xaa,0x00,
  0x2a,0xaa,0x00,0x55,0xaa,0x00,0x7f,0xaa,0x00,0xaa,0xaa,0x00,0xd4,0xaa,0x2a,0x00,
  0xaa,0x2a,0x2a,0xaa,0x2a,0x55,0xaa,0x2a,0x7f,0xaa,0x2a,0xaa,0xaa,0x2a,0xd4,0xaa,
  0x55,0x00,0xaa,0x55,0x2a,0xaa,0x55,0x55,0xaa,0x55,0x7f,0xaa,0x55,0xaa,0xaa,0x55,
  0xd4,0xaa,0x7f,0x00,0xaa,0x7f,0x2a,0xaa,0x7f,0x55,0xaa,0x7f,0x7f,0xaa,0x7f,0xaa,
  0xaa,0x7f,0xd4,0xaa,0xaa,0x00,0xaa,0xaa,0x2a,0xaa,0xaa,0x55,0xaa,0xaa,0x7f,0xaa,
  0xaa,0xaa,0xaa,0xaa,0xd4,0xaa,0xd4,0x00,0xaa,0xd4,0x2a,0xaa,0xd4,0x55,0xaa,0xd4,
  0x7f,0xaa,0xd4,0xaa,0xaa,0xd4,0xd4,0xd4,0x00,0x00,0xd4,0x00,0x2a,0xd4,0x00,0x55,
  0xd4,0x00,0x7f,0xd4,0x00,0xaa,0xd4,0x00,0xd4,0xd4,0x2a,0x00,0xd4,0x2a,0x2a,0xd4,
  0x2a,0x55,0xd4,0x2a,0x7f,0xd4,0x2a,0xaa,0xd4,0x2a,0xd4,0xd4,0x55,0x00,0xd4,0x55,
  0x2a,0xd4,0x55,0x55,0xd4,0x55,0x7f,0xd4,0x55,0xaa,0xd4,0x55,0xd4,0xd4,0x7f,0x00,
  0xd4,0x7f,0x2a,0xd4,0x7f,0x55,0xd4,0x7f,0x7f,0xd4,0x7f,0xaa,0xd4,0x7f,0xd4,0xd4,
  0xaa,0x00,0xd4,0xaa,0x2a,0xd4,0xaa,0x55,0xd4,0xaa,0x7f,0xd4,0xaa,0xaa,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0xd4,0xd4,0x2a,0xd4,0xd4,0x55,0xd4,0xd4,0x7f,0xd4,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0x00,0x00,0x0c,0x0c,0x0c,0x19,0x19,0x19,0x26,0x26,0x26,0x33,
  0x33,0x33,0x3f,0x3f,0x3f,0x4c,0x4c,0x4c,0x59,0x59,0x59,0x66,0x66,0x66,0x72,0x72,
  0x72,0x7f,0x7f,0x7f,0x8c,0x8c,0x8c,0x99,0x99,0x99,0xa5,0xa5,0xa5,0xb2,0xb2,0xb2,
  0xbf,0xbf,0xbf,0xcc,0xcc,0xcc,0xd8,0xd8,0xd8,0xe5,0xe5,0xe5,0xf2,0xf2,0xf2,0xff,
  0xfb,0xf0,0xa0,0xa0,0xa4,0x80,0x80,0x80,0xff,0x00,0x00,0x00,0xff,0x00,0xff,0xff,
  0x00,0x00,0x00,0xff,0xff,0x00,0xff,0x00,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x08,0xfe,0x00,0xf1,0x09,0x1c,0x48,0xb0,0xa0,
  0xc1,0x83,0x08,0x13,0x2a,0x5c,0xc8,0xb0,0xa1,0xc3,0x87,0x10,0x23,0x4a,0x9c,0x48,
  0xb1,0xa2,0xc5,0x8b,0x18,0x33,0x6a,0xdc,0xc8,0xb1,0xa3,0xc7,0x8f,0x20,0x43,0x8a,
  0x1c,0x49,0xb2,0xa4,0xc9,0x93,0x28,0x53,0xaa,0x5c,0xc9,0xb2,0xa5,0xcb,0x97,0x30,
  0x5b,0x1e,0x98,0x49,0xb3,0xa6,0xcd,0x9b,0x38,0x73,0xea,0xd4,0xa9,0x70,0xa7,0xcf,
  0x9f,0x40,0x73,0xf6,0x0c,0x4a,0xb4,0xa8,0xd0,0x84,0x46,0x93,0x2a,0x1d,0xaa,0xb4,
  0x29,0x50,0xa6,0x4e,0xa3,0x1e,0x45,0x38,0x33,0xe6,0xbf,0xaa,0x48,0x0f,0xe0,0x03,
  0xc0,0xb5,0xab,0xd7,0xaf,0x60,0xc3,0x8a,0x1d,0xcb,0x35,0xdc,0x55,0xad,0x59,0xb7,
  0x86,0x5b,0xcb,0xb6,0xad,0x5b,0xb6,0xff,0xe2,0xca,0x8d,0x0b,0xe0,0xad,0x5d,0xbb,
  0x67,0x87,0xaa,0xbd,0xcb,0xf7,0xdf,0xdb,0xbd,0x7c,0xfb,0x62,0xa5,0xaa,0xb5,0x6e,
  0x60,0xbc,0x7f,0x0d,0x1f,0xc6,0x3b,0xf8,0x60,0x55,0x00,0x12,0xfd,0xba,0xdd,0x1a,
  0xd9,0x6c,0x63,0x83,0x8f,0xb7,0x92,0x05,0x2b,0xb9,0xad,0xe6,0xcd,0x5f,0xcd,0x5a,
  0x46,0x4b,0x98,0x32,0x80,0xb9,0xa8,0x53,0xc7,0x4d,0xac,0xba,0xf5,0x6a,0xbf,0x79,
  0xd3,0x42,0x5e,0xec,0xb6,0x33,0x5b,0xc0,0xb4,0xd7,0xc2,0xbe,0x5c,0xf0,0xb1,0xeb,
  0xdf,0xac,0x7f,0xb7,0x1e,0xad,0x57,0x71,0x6e,0xd1,0x93,0x8d,0x1f,0x27,0x2e,0x7b,
  0xb9,0xee,0x83,0xca,0x8f,0xc7,0x2e,0x1d,0x7d,0xf1,0x58,0xe7,0x70,0x79,0x13,0xac,
  0x8a,0xbd,0xfb,0xe1,0xe9,0x8e,0x0f,0x35,0x08,0x1f,0x4f,0xbe,0xbc,0xf6,0x81,0x52,
  0xd3,0xe3,0x84,0xaa,0x5e,0x3d,0xfb,0xf6,0x52,0xdf,0xc3,0x77,0x2a,0x7f,0xfe,0xd2,
  0xac,0xf6,0xe3,0xc7,0xdc,0xcf,0xbf,0xbf,0xff,0xff,0x00,0x06,0x28,0xe0,0x80,0x04,
  0x16,0x68,0xe0,0x81,0x08,0x26,0xa8,0xe0,0x82,0x0c,0xfe,0x17,0x10,0x00,0x3b
  };

CDispenserSlider::CDispenserSlider()
{
	m_Properties.append(new CPropertyLayout<FXSlider>(this));
	m_Properties.append(new CPropertyFrame<FXSlider>(this));
	m_Properties.append(new CPropertySliderStyle<FXSlider>(this));
	m_Properties.append(new CPropertyBackColor<FXSlider, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertySlotColor<FXSlider, FXRGB(255, 255, 255)>(this));
	m_Properties.append(new CPropertyTipText<FXSlider>(this));
//	m_Properties.append(new CPropertyRange<FXSlider>(this));
	m_Properties.append(new CPropertyPadTop<FXSlider>(this));
	m_Properties.append(new CPropertyPadLeft<FXSlider>(this));
	m_Properties.append(new CPropertyPadBottom<FXSlider>(this));
	m_Properties.append(new CPropertyPadRight<FXSlider>(this));
	m_Properties.append(new CPropertyLeft<FXSlider>(this));
	m_Properties.append(new CPropertyTop<FXSlider>(this));
	m_Properties.append(new CPropertyWidth<FXSlider>(this));
	m_Properties.append(new CPropertyHeight<FXSlider>(this));
}

const char* CDispenserSlider::getWidgetName(void)
{
	return "FXSlider";
}

const void* CDispenserSlider::getWidgetIcon(void)
{
	return Slider;
}

const char* CDispenserSlider::getWidgetCategory(void)
{
	return "Extended";
}

const int   CDispenserSlider::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserSlider::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserSlider::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserSlider::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserSlider::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserSlider::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserSlider::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserSlider::getClassInfo(void)
{
	return &FXSlider::metaClass;
}

FXbool CDispenserSlider::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserSlider::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserSlider::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserSlider::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXSlider::metaClass);
}

IDispenser* CDispenserSlider::clone(void)
{
	return new CDispenserSlider;
}

FXWindow* CDispenserSlider::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXSlider(p, tgt, sel,
				FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserSlider::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXSlider(" << parent << ", 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}
