/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

/*
 *  This file contains the source code for the Outlook style shutter bar.
 *  
 *  CButton models the button that represents a widget the user can add.
 *  A number of mouse events are handled to modify the appearance of the
 *  button.
 *
 *  CShutter models the shutter widget. CShutter overrides FXShutter in
 *  order to set the widget width.
 *
 *  COutlookButton models each shutter item's button -- the button that
 *  appears at the top or bottom of the shutter. COutlookButton overrides
 *  CButton::onPaint in order to prevent drawing the focus rectangle.
 *
 *  CShutterItem represents each shutter 'pane'. CShutterItem overrides
 *  FXShutterItem in order to contain a COutlookButton rather than a 
 *  CButton.
 */

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CConnector.h>

#include <COutlookBar.h>

///////////////////////////////////////////////////////////////////////////////
//

FXDEFMAP(CButton) CButtonMap[]=
{
	FXMAPFUNC(SEL_PAINT,0,CButton::onPaint),
	FXMAPFUNC(SEL_MOTION,0,CButton::onMotion),
	FXMAPFUNC(SEL_LEAVE,0,CButton::onLeave),
	FXMAPFUNC(SEL_DEACTIVATE,0,CButton::onDeactivate)
};

FXIMPLEMENT(CButton, FXButton, CButtonMap, ARRAYNUMBER(CButtonMap)) 

CButton::CButton(FXComposite* p, CConnector& connector, IDispenser* pDispenser, FXObject* tgt, FXSelector sel)
: FXButton	(
			 p, 
			 pDispenser->getWidgetName(), 
			 new FXGIFIcon(p->getApp(),pDispenser->getWidgetIcon(),FXRGB(192,192,192),IMAGE_OPAQUE|IMAGE_ALPHAGUESS),
			 tgt, 
			 sel,
			 FRAME_RAISED|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_CENTER_X|ICON_ABOVE_TEXT|JUSTIFY_CENTER_X,2,2,2,2,2,2,2,2
			 )
{
	m_Connector = connector;

	m_Connector.pDispenser   = pDispenser;
	m_Connector.pButton      = this;
	m_Connector.bDoubleClick = FALSE;
}

CButton::~CButton()
{
	/*
	Each button has an associated connector. The CConnector was created
	in CMainFrame::CMainFrame and passed to CButtons c-tor.  When the
	button is clicked the pDispenser is cloned in CConnector::onAddWidget.
	Deleting the clone is the responsibility of CApp but deleting the 
	original is done here.
	*/

	if ( m_Connector.pDispenser )
	{
		delete m_Connector.pDispenser;
		m_Connector.pDispenser = 0;
	}

    delete icon; // Created in c-tor -- see above
}

long CButton::onDeactivate(FXObject*,FXSelector,void* ptr)
{
	FXEvent* event=(FXEvent*)ptr;
	FXuint click=(state==STATE_DOWN);
	flags&=~FLAG_PRESSED;
	flags|=FLAG_UPDATE;

	if ( click )
	{
		if ( event->click_count == 2 )
		{
			m_Connector.bDoubleClick = TRUE;
		}
		else
		{
			m_Connector.bDoubleClick = FALSE;
		}

		handle(this,MKUINT(0,SEL_COMMAND),(void*)&m_Connector);
	}

  FXint tw=0,th=0,iw=0,ih=0,tx,ty,ix,iy;

  if(!label.empty()){
    tw=labelWidth(label);
    th=labelHeight(label);
    }
  if(icon){
    iw=icon->getWidth();
    ih=icon->getHeight();
    }
  just_x(tx,ix,tw,iw);
  just_y(ty,iy,th,ih);

	FXDCWindow dc(this);

        dc.setForeground(backColor);
	   setShadowColor(FXRGB(0,0,0));
        drawSunkenRectangle(dc,ix-1,iy-1,iw+2,ih+2);

	  return 1;
}

long CButton::onPaint(FXObject*,FXSelector,void* ptr){
  FXint tw=0,th=0,iw=0,ih=0,tx,ty,ix,iy;
  FXEvent *ev=(FXEvent*)ptr;
  
  // Start drawing
  FXDCWindow dc(this,ev);
  
    if(isEnabled() && (state==STATE_ENGAGED)){
      dc.setForeground(hiliteColor);
      dc.fillRectangle(0,0,width,height);
      }
    else{
      dc.setForeground(FXRGB(128,128,128));
      dc.fillRectangle(0,0,width,height);
      }
  
  // Place text & icon
  if(!label.empty()){
    tw=labelWidth(label);
    th=labelHeight(label);
    }
  if(icon){
    iw=icon->getWidth();
    ih=icon->getHeight();
    }
  just_x(tx,ix,tw,iw);
  just_y(ty,iy,th,ih);
  
  // Shift a bit when pressed
  if(state && (options&(FRAME_RAISED|FRAME_SUNKEN))){ /*++tx; ++ty;*/ ++ix; ++iy; }

  // Draw the icon
  if(icon){
    if(isEnabled())
      dc.drawIcon(icon,ix,iy);
    else
      dc.drawIconSunken(icon,ix,iy);
    }
  
  // Draw the text
  if(!label.empty()){
    dc.setTextFont(font);
    if(isEnabled()){
      dc.setForeground(FXRGB(255,255,255));
      drawLabel(dc,label,hotoff,tx,ty,tw,th);
      }
    else{
      dc.setForeground(hiliteColor);
      drawLabel(dc,label,hotoff,tx+1,ty+1,tw,th);
      dc.setForeground(shadowColor);
      drawLabel(dc,label,hotoff,tx,ty,tw,th);
      }
    }
  return 1;
  }

long CButton::onMotion(FXObject*,FXSelector,void* ptr)
{
  FXint tw=0,th=0,iw=0,ih=0,tx,ty,ix,iy;

  if(!label.empty()){
    tw=labelWidth(label);
    th=labelHeight(label);
    }
  if(icon){
    iw=icon->getWidth();
    ih=icon->getHeight();
    }
  just_x(tx,ix,tw,iw);
  just_y(ty,iy,th,ih);


      if(isEnabled() && underCursor() && (state==STATE_UP))
	  {
 		FXDCWindow dc(this);
       dc.setForeground(backColor);
	   setShadowColor(FXRGB(0,0,0));
        drawRaisedRectangle(dc,ix-1,iy-1,iw+2,ih+2);
      }
      else if(isEnabled() && underCursor() && (state==STATE_DOWN))
	  {
 		FXDCWindow dc(this);
        dc.setForeground(backColor);
	   setShadowColor(FXRGB(0,0,0));
        drawSunkenRectangle(dc,ix-1,iy-1,iw+2,ih+2);
	  }

	  return 1;
      
}

long CButton::onLeave(FXObject* sender,FXSelector sel,void* ptr){
  FXint tw=0,th=0,iw=0,ih=0,tx,ty,ix,iy;

  if(!label.empty()){
    tw=labelWidth(label);
    th=labelHeight(label);
    }
  if(icon){
    iw=icon->getWidth();
    ih=icon->getHeight();
    }
  just_x(tx,ix,tw,iw);
  just_y(ty,iy,th,ih);

	FXDCWindow dc(this);

      if(isEnabled() && underCursor() && (state==STATE_DOWN))
	  {
        dc.setForeground(backColor);
	   setShadowColor(FXRGB(0,0,0));
        drawSunkenRectangle(dc,ix-1,iy-1,iw+2,ih+2);
	  }
	  else
	  {
         dc.setForeground(FXRGB(128,128,128));
		  dc.drawRectangle(ix-1,iy-1,iw+1,ih+1);
	  }

	  return 1;
  }

CShutter::CShutter(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs)
: FXShutter(p,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb,hs,vs)
{
} 

FXint CShutter::getDefaultWidth()
{
	return 120;
}

///////////////////////////////////////////////////////////////////////////////
//

FXDEFMAP(COutlookButton) COutlookButtonMap[]=
{
	FXMAPFUNC(SEL_PAINT,0,COutlookButton::onPaint)
};

FXIMPLEMENT(COutlookButton, FXButton, COutlookButtonMap, ARRAYNUMBER(COutlookButtonMap)) 

COutlookButton::COutlookButton()
: FXButton()
{
}

COutlookButton::COutlookButton(FXComposite* p,const FXString& text,FXIcon* ic,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb)
: FXButton(p,text,ic,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb)
{
}

long COutlookButton::onPaint(FXObject*,FXSelector,void* ptr){
  FXint tw=0,th=0,iw=0,ih=0,tx,ty,ix,iy;
  FXEvent *ev=(FXEvent*)ptr;
  
  // Start drawing
  FXDCWindow dc(this,ev);
  
  // Got a border at all?
  if(options&(FRAME_RAISED|FRAME_SUNKEN)){

    // Toolbar style
    if(options&BUTTON_TOOLBAR){
      
      // Enabled and cursor inside, and up
      if(isEnabled() && underCursor() && (state==STATE_UP)){
        dc.setForeground(backColor);
        dc.fillRectangle(border,border,width-border*2,height-border*2);
        if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,0,0,width,height);
        else drawRaisedRectangle(dc,0,0,width,height);
        }
      
      // Enabled and cursor inside and down
      else if(isEnabled() && underCursor() && (state==STATE_DOWN)){
        dc.setForeground(backColor);
        dc.fillRectangle(border,border,width-border*2,height-border*2);
        if(options&FRAME_THICK) drawDoubleSunkenRectangle(dc,0,0,width,height);
        else drawSunkenRectangle(dc,0,0,width,height);
        }
      
      // Enabled and checked
      else if(isEnabled() && (state==STATE_ENGAGED)){
        dc.setForeground(hiliteColor);
        dc.fillRectangle(border,border,width-border*2,height-border*2);
        if(options&FRAME_THICK) drawDoubleSunkenRectangle(dc,0,0,width,height);
        else drawSunkenRectangle(dc,0,0,width,height);
        }
        
      // Disabled or unchecked or not under cursor
      else{
        dc.setForeground(backColor);
        dc.fillRectangle(0,0,width,height);
        }
      }
 
    // Normal style
    else{
      
      // Default
      if(isDefault()){
        
        // Draw in up state if disabled or up
        if(!isEnabled() || (state==STATE_UP)){
          dc.setForeground(backColor);
          dc.fillRectangle(border+1,border+1,width-border*2-1,height-border*2-1);
          if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,1,1,width-1,height-1);
          else drawRaisedRectangle(dc,1,1,width-1,height-1);
          }
        
        // Draw sunken if enabled and either checked or pressed
        else{
          if(state==STATE_ENGAGED) dc.setForeground(hiliteColor); else dc.setForeground(backColor);
          dc.fillRectangle(border,border,width-border*2-1,height-border*2-1);
          if(options&FRAME_THICK) drawDoubleSunkenRectangle(dc,0,0,width-1,height-1);
          else drawSunkenRectangle(dc,0,0,width-1,height-1);
          }
        
        // Black default border
        drawBorderRectangle(dc,0,0,width,height);
        }
      
      // Non-Default
      else{
        
        // Draw in up state if disabled or up
        if(!isEnabled() || (state==STATE_UP)){
          dc.setForeground(backColor);
          dc.fillRectangle(border,border,width-border*2,height-border*2);
          if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,0,0,width,height);
          else drawRaisedRectangle(dc,0,0,width,height);
          }
        
        // Draw sunken if enabled and either checked or pressed
        else{
          if(state==STATE_ENGAGED) dc.setForeground(hiliteColor); else dc.setForeground(backColor);
          dc.fillRectangle(border,border,width-border*2,height-border*2);
          if(options&FRAME_THICK) drawDoubleSunkenRectangle(dc,0,0,width,height);
          else drawSunkenRectangle(dc,0,0,width,height);
          }
        }
      }
    }
  
  // No borders
  else{
    if(isEnabled() && (state==STATE_ENGAGED)){
      dc.setForeground(hiliteColor);
      dc.fillRectangle(0,0,width,height);
      }
    else{
      dc.setForeground(backColor);
      dc.fillRectangle(0,0,width,height);
      }
    }
  
  // Place text & icon
  if(!label.empty()){
    tw=labelWidth(label);
    th=labelHeight(label);
    }
  if(icon){
    iw=icon->getWidth();
    ih=icon->getHeight();
    }
  just_x(tx,ix,tw,iw);
  just_y(ty,iy,th,ih);
  
  // Shift a bit when pressed
  if(state && (options&(FRAME_RAISED|FRAME_SUNKEN))){ ++tx; ++ty; ++ix; ++iy; }

  // Draw the icon
  if(icon){
    if(isEnabled())
      dc.drawIcon(icon,ix,iy);
    else
      dc.drawIconSunken(icon,ix,iy);
    }
  
  // Draw the text
  if(!label.empty()){
    dc.setTextFont(font);
    if(isEnabled()){
      dc.setForeground(textColor);
      drawLabel(dc,label,hotoff,tx,ty,tw,th);
//      if(hasFocus()){
//        drawFocusRectangle(dc,border+2,border+2,width-2*border-4,height-2*border-4);
//        }
      }
    else{
      dc.setForeground(hiliteColor);
      drawLabel(dc,label,hotoff,tx+1,ty+1,tw,th);
      dc.setForeground(shadowColor);
      drawLabel(dc,label,hotoff,tx,ty,tw,th);
      }
    }
  return 1;
  }

///////////////////////////////////////////////////////////////////////////////
//

FXIMPLEMENT(CShutterItem, FXShutterItem, 0, 0) 

CShutterItem::CShutterItem()
:FXShutterItem()
{
}

CShutterItem::CShutterItem(FXShutter *p,const char* text,FXIcon* icon,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs)
: FXShutterItem(p, text, icon, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
{
	delete button;
	button=new COutlookButton(this,text,icon,this,FXShutterItem::ID_SHUTTERITEM_BUTTON,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT);
}

