/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>
#include "fxkeys.h"

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CConnector.h>

#include "CTracker.h"
#include "CTrackManager.h"

#include <CApp.h>
#include "CAppDialog.h"

#include <CPropertyList.h>

#include <CComboBox.h>
#include <CBoolComboBox.h>
#include <CCheckComboBox.h>
#include <CColorComboBox.h>

#include "CVariant.h"

///////////////////////////////////////////////////////////////////////////////
//

FXDEFMAP(CTextField) CTextFieldMap[]=
{
	FXMAPFUNC(SEL_KEYPRESS,0,CTextField::onKeyPress),
	FXMAPFUNC(SEL_CLOSED,0,CTextField::onClose)
};

FXIMPLEMENT(CTextField, FXTextField, CTextFieldMap, ARRAYNUMBER(CTextFieldMap)) 

CTextField::CTextField(FXComposite* p, CConnector connector, FXString strProperty)
	: FXTextField(p,10,NULL,0,LAYOUT_FILL_X|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT)
{
	m_Connector = connector;
	m_bHadFocus = FALSE;

	application->preventDelete();
}

CTextField::~CTextField()
{
	if ( application->isRunning() )
	{
		updateWidget(FALSE);
	}

	application->allowDelete();
}

void CTextField::updateWidget(FXbool bIsWindowRunning)
{
	// This may seem a bit odd but is very important. CConnector::set
	// will set the property but will also validate the widget dimensions
	// using the appropriate CDispenser. If this text field is being used
	// for a dimension and an illegal value is entered then CConnector::set
	// will set the value to something other than the value in this
	// text field. The call to CConnector::get is required to synchronise
	// the value in this text field with the actual widget value in this situation.
	m_Connector.set(getText());
	setText(m_Connector.get());
}

void CTextField::setFocus()
{
	FXTextField::setFocus();

	handle(this,MKUINT(0,SEL_FOCUSIN),NULL);

	if ( !m_bHadFocus)
	{
		handle(this,MKUINT(FXTextField::ID_SELECT_ALL,SEL_COMMAND),NULL);
		m_bHadFocus = TRUE;
	}

	flags|=FLAG_HELP;
}

long CTextField::onKeyPress(FXObject* o, FXSelector s,void* ptr)
{
	long lReturn = FXTextField::onKeyPress(o, s, ptr);

	if ( m_Connector.pProperty->style() & PROPERTY_UPDATEEVERYKEY ) 
	{
		updateWidget();
	}

	return lReturn;
}

long CTextField::onClose(FXObject* o, FXSelector s,void* ptr)
{
	updateWidget();

	return 1;
}

///////////////////////////////////////////////////////////////////////////////
//

#define SELECT_MASK (LIST_SINGLESELECT|LIST_BROWSESELECT)

FXIMPLEMENT(CPropertyItem,FXListItem,NULL,0)

CPropertyItem::CPropertyItem()
{
	m_pChildWidget = 0;
	m_pList = 0;
	m_type = PROPERTY_NONE;
}

CPropertyItem::CPropertyItem(CConnector connector)
{
	m_pChildWidget = 0;

	m_Connector = connector;

	// Give property a chance to enable/disable other properties
	m_Connector.pProperty->validate(&m_Connector);

	label  = m_Connector.pProperty->name();
	m_type = m_Connector.pProperty->type();
	tip    = m_Connector.pProperty->tooltip();

	m_pList = 0;

	refreshProperty();
}

CPropertyItem::~CPropertyItem()
{
	if ( m_pChildWidget  )
	{
		m_pChildWidget->destroy();
		delete m_pChildWidget;
		m_pChildWidget = 0;
	}
}

void CPropertyItem::refreshProperty()
{
	switch (m_type)
	{
	case PROPERTY_NONE:
		m_strProperty = "!UNKNOWN!";
		break;

	case PROPERTY_COMBO:
		m_strProperty = "(Select)";
		break;

	case PROPERTY_LIST:
		m_strProperty = "(List)";
		break;

	case PROPERTY_COLOR:
		m_strProperty = m_Connector.pProperty->get(m_Connector.pWidget);
		m_Color       = getColorFromString(m_strProperty);
		break;

	case PROPERTY_TEXT:
	case PROPERTY_BOOL:
		m_strProperty = m_Connector.get();
		break;
	};
}

void CPropertyItem::refreshDescription()
{
	m_Connector.pDescription->setText(m_Connector.pProperty->description());
}

void CPropertyItem::draw(const FXList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h)
{
	m_x=x;
	m_y=y;
	m_w=w;
	m_h=h;
	m_pList=(FXList*)list;

	// Draw the text in the two columns
	drawColumn('L', label, list, dc, x,y,w/2,h, isSelected());        // LH Column

	// -1 to align with text in a child widget
	drawColumn('R', m_strProperty, list, dc, x+w/2-1,y,w-(w/2)+1,h, FALSE); // RH Column

	// Draw the column borders
	FXColor grey  = FXRGB(192,192,192);
	dc.setForeground(grey);

//	dc.drawLine(x+w-1,0,x+w-1,y+h-1);         // Right
	dc.drawLine(0,y+h-1,x+w-1,y+h-1);         // Bottom
	dc.drawLine(x+(w/2)-1,0,x+(w/2)-1,y+h-1); // Middle

    dc.setForeground(list->getBackColor());
}

// This code is FXListItem::draw with FXString added
// and drawFocus calls removed
void CPropertyItem::drawColumn(FXchar column, FXString& s,const FXList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h, FXbool bHighlight){
  FXint ih=0,th=0;
  if(icon) ih=icon->getHeight();
  if(s.text()) th=list->getFont()->getFontHeight();
  if(bHighlight){
    dc.setForeground(list->getSelBackColor());
    dc.fillRectangle(x,y,w,h);
    }
  else
  {
	if ( !m_Connector.pProperty->isEnabled() && column == 'R' )
		dc.setForeground(FXRGB(128,128,128));
	else
//		dc.setForeground(list->getBackColor());
		dc.setForeground(FXRGB(255,255,255));

    dc.fillRectangle(x,y,w,h);
  }
//  if(hasFocus()){
//    drawFocus(list,dc,x,y,w,h);
//    }
  x+=SIDE_SPACING/2;
  if(icon){
    dc.drawIcon(icon,x,y+(h-ih)/2);
    x+=ICON_SPACING+icon->getWidth();
    }
  if(s.text()){
    dc.setTextFont(list->getFont());
    if(!isEnabled())
      dc.setForeground(makeShadowColor(list->getBackColor()));
    else if(bHighlight)
      dc.setForeground(list->getSelTextColor());
    else
      dc.setForeground(list->getTextColor());
		if (m_type == PROPERTY_COLOR && column == 'R')
		{
//			FXchar pszColor[12];
//			sprintf(pszColor,"#%02x%02x%02x",FXREDVAL(m_Color),FXGREENVAL(m_Color),FXBLUEVAL(m_Color));
//			s = pszColor;
//			s.upper();
		
		    dc.drawText(x+th+1,y+(h-th)/2+list->getFont()->getFontAscent(),s.text(),s.length());

			dc.setForeground(FXRGB(0,0,0));
			dc.drawRectangle(x+1,y+3,th-3,th-3);
			dc.setForeground(m_Color);
			dc.fillRectangle(x+2,y+4,th-4,th-4);
		}
		else
		{
		    dc.drawText(x,y+(h-th)/2+list->getFont()->getFontAscent(),s.text(),s.length());
		}
    }
  }

FXObject* CPropertyItem::invokeChildWidget(void)
{
	// CMW 18.10.00 Added this m_pList test for safety
	if ( m_pChildWidget==0 && m_pList )
	{
		if ( !m_Connector.pProperty->isEnabled() )
			return m_pChildWidget;

		switch (m_type)
		{
		case PROPERTY_COMBO:
			m_pChildWidget=new CCheckComboBox(&m_Connector, (FXComposite*)m_pList, /*DDWidth*/50, /*DDHeight*/4, (FXComposite*)m_pList, 0,
				COMBOBOX_STATIC|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,0,0,10,10,10,10);
			break;

		case PROPERTY_COLOR:
			m_strProperty = m_Connector.pProperty->get(m_Connector.pWidget);
			m_Color       = getColorFromString(m_strProperty);

			m_pChildWidget=new CColorComboBox(&m_Connector, (FXComposite*)m_pList, /*DDWidth*/50, /*DDHeight*/4, (FXComposite*)m_pList, 0,
				COMBOBOX_STATIC|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,0,0,10,10,10,10);

			((CColorComboBox*)m_pChildWidget)->setSelectedColor(m_Color);
			break;

		case PROPERTY_TEXT:
			m_pChildWidget=new CTextField(m_pList, m_Connector, label);
			((FXTextField*)m_pChildWidget)->setText(m_strProperty);
			m_pList->setFocus();
			break;

		case PROPERTY_BOOL:
			m_pChildWidget=new CBoolComboBox(&m_Connector, (FXComposite*)m_pList, /*DDWidth*/50, /*DDHeight*/4, (FXComposite*)m_pList, 0,
				COMBOBOX_STATIC|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,0,0,10,10,10,10);
			break;

		case PROPERTY_LIST:
			m_pChildWidget=new CListComboBox(&m_Connector, (FXComposite*)m_pList, /*DDWidth*/50, /*DDHeight*/4, (FXComposite*)m_pList, 0,
				COMBOBOX_STATIC|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,0,0,10,10,10,10);
			break;

		case PROPERTY_NONE:
			m_pChildWidget = 0;
		};

		if ( m_pChildWidget )
		{
			m_pChildWidget->create();
			m_pChildWidget->position(m_x+(m_w/2), m_y, /*m_w/2*/m_w-(m_w/2), m_h-1);
		}
	}

	return m_pChildWidget;
}

void CPropertyItem::closeChildWidget(void)
{
	if ( m_pChildWidget  )
	{
		switch (m_type)
		{
		case PROPERTY_COMBO:
			m_strProperty = "(Select)";
			break;

		case PROPERTY_LIST:
			m_strProperty = "(List)";
			break;

		case PROPERTY_COLOR:
			m_strProperty = m_Connector.pProperty->get(m_Connector.pWidget);
			m_Color       = getColorFromString(m_strProperty);
			break;

		case PROPERTY_TEXT:
			m_strProperty = ((FXTextField*)m_pChildWidget)->getText();
			break;

		case PROPERTY_BOOL:
			m_strProperty = m_Connector.pProperty->get(m_Connector.pWidget);
			break;

		case PROPERTY_NONE:
			m_strProperty = "";
		};

		if ( m_pChildWidget )
		{
			delete m_pChildWidget;
			m_pChildWidget = 0;
		}
	}
}

void CPropertyItem::moveChildWidget(FXint y, FXint w)
{
	m_w = w;
	m_y = y;
	if ( m_pChildWidget  )
	{
		m_pChildWidget->position(m_x+(m_w/2), m_y, /*m_w/2*/m_w-(m_w/2), m_h-1);
	}
}

void CPropertyItem::setChildWidget(FXint y)
{
	m_y = y;
	if ( m_pChildWidget  )
	{
		m_pChildWidget->position(m_x+(m_w/2), m_y, /*m_w/2*/m_w-(m_w/2), m_h-1);
	}
}

///////////////////////////////////////////////////////////
//

/*
 * The CPropertyList class provides an implementation of a list
 * with two columns: The left-hand column displays the property
 * label and the right hand column displays the property value.
 * 	
 * When the user selects a row CPropertyList creates a child widget
 * in the value column to allow the user to change the value.
 */

FXDEFMAP(CPropertyList) CPropertyListMap[]=
{
//	FXMAPFUNC(SEL_CHANGED,0,CPropertyList::onChanged),
	FXMAPFUNC(SEL_COMMAND,FXWindow::ID_VSCROLLED,CPropertyList::onVScrollerChanged),
	FXMAPFUNC(SEL_CHANGED,FXWindow::ID_VSCROLLED,CPropertyList::onVScrollerDragged),
	FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,CPropertyList::onLeftBtnPress),
	FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0,CPropertyList::onLeftBtnRelease),
	FXMAPFUNC(SEL_MOTION,0,CPropertyList::onMotion),
	FXMAPFUNC(SEL_KEYPRESS,0,CPropertyList::onKeyPress),
	FXMAPFUNC(SEL_KEYRELEASE,0,CPropertyList::onKeyRelease),
	FXMAPFUNC(SEL_ENTER,0,CPropertyList::onEnter),
	FXMAPFUNC(SEL_LEAVE,0,CPropertyList::onLeave),
	FXMAPFUNC(SEL_FOCUSIN,0,CPropertyList::onFocusIn),
	FXMAPFUNC(SEL_FOCUSOUT,0,CPropertyList::onFocusOut),
	FXMAPFUNC(SEL_UPDATE,FXWindow::ID_QUERY_TIP,CPropertyList::onQueryTip)
};

FXIMPLEMENT(CPropertyList, FXList, CPropertyListMap, ARRAYNUMBER(CPropertyListMap)) 

CPropertyList::CPropertyList(FXComposite* p,FXuint opts)
: FXList(p,1,NULL,0,opts,0,0,0,0)
{
	flags|=FLAG_ENABLED|FLAG_SHOWN|LIST_BROWSESELECT;
	setBackColor(FXRGB(128,128,128));
	m_pCurrentItem = 0;
}

CPropertyList::~CPropertyList()
{
}

/*
 * The widget has been resized. This may have caused the vertical 
 * scroll bar to appear or disappear which means that the child
 * widget needs resizing if one is active.
 */
void CPropertyList::layout()
{
	FXList::layout();

	FXint w = viewport_w;

	register FXint i;
	for(i=0; i<nitems; i++)
	{
		((CPropertyItem*)items[i])->moveChildWidget(i * itemHeight + pos_y, w);
	}
}

void CPropertyList::refreshProperty()
{
	for(FXint i=0; i<nitems; i++)
	{
		((CPropertyItem*)items[i])->refreshProperty();
	}
	update();
}

void CPropertyList::clearItems()
{
	FXList::clearItems();
	target = 0;
	m_pCurrentItem = 0;
}

void CPropertyList::setup(CConnector& connector, FXbool bMultiple)
{
	clearItems();
	
	FXint nCount = connector.pDispenser->getPropertyCount();
	for (FXint i = 0; i < nCount; i++)
	{
		IProperty* pProperty = connector.pDispenser->getProperty(i);

		FXbool bAddProperty = TRUE;

		if ( bMultiple )
		{
			if ( i == 0 || (pProperty->style() & PROPERTY_MULTIPLE) == 0 )
			{
				// i == 0 - The Name property. 
				// Do not add this during multiple selection
				bAddProperty = FALSE;
			}
			else
			{
				FXint nCount = connector.pTrackManager->getTrackerCount();

				for ( FXint j = 0; j < nCount; j++ )
				{
					CConnector* pConnector = connector.pTrackManager->getConnector(j);

					if ( pConnector == &connector )
						continue;

					bAddProperty = FALSE; 

					FXint nC = pConnector->pDispenser->getPropertyCount();
					for (FXint k = 1; k < nC; k++)
					{
						IProperty* pP = pConnector->pDispenser->getProperty(k);

						if ( pProperty->name() == pP->name() )
							bAddProperty = TRUE;
					}
				}
			}
		}

		if ( bAddProperty )
		{
			CConnector c = connector;
			c.pProperty  = pProperty;

			CPropertyItem* p = new CPropertyItem( c );
			connector.pProperties->appendItem(p);
		}
	}

	if ( nCount )
	{
		selectItem(0);
	}
}

FXbool CPropertyList::selectItem(FXint index)
{
	if ( index != -1 && index != current )
	{
		CPropertyItem* pCurrent = (CPropertyItem*)retrieveItem(index);
		pCurrent->refreshDescription();
	}
	return FXList::selectItem(index);
}

void CPropertyList::close(void)
{
	CPropertyItem* pCurrent = (CPropertyItem*)retrieveItem(getCurrentItem());
	pCurrent->close();
	current = -1;
}

/*
 * A new property has been selected. If a child widget is active
 * then remove it. Then add a new child widget for the newly
 * selected property.
 */

//long CPropertyList::onChanged(FXObject* o,FXSelector s,void* ptr)
//{
//	FXint index = (FXint)(long)ptr;
//
//	if ( m_pCurrentItem /*(index == -1 || index != current) && current != -1*/ )
//	{
//		m_pCurrentItem->closeChildWidget();
//		m_pCurrentItem = 0;
//		target = NULL;
//	}
//
//	if ( index != -1 /*&& index != current*/ )
//	{
//		/*CPropertyItem**/ m_pCurrentItem = (CPropertyItem*)retrieveItem(index);
//		/*target =*/ m_pCurrentItem->invokeChildWidget();
//		m_pCurrentItem->refreshDescription();
//	}
//
//	return 1;//FXList::onChanged(o, s, ptr);
//}

/*
 * The user has moved the scroll bar and has moved the position
 * of the property item. The associated child widget must also
 * be moved if one is active.
 */
long CPropertyList::onVScrollerChanged(FXObject* o,FXSelector s,void* ptr)
{
	FXList::onVScrollerChanged(o, s, ptr);

	if ( current != -1 )
	{
		((CPropertyItem*)items[current])->setChildWidget(current * itemHeight + pos_y);
	}

	return 1;
}

/*
 * The user has moved the scroll bar and has moved the position
 * of the property item. The associated child widget must also
 * be moved if one is active.
 */
long CPropertyList::onVScrollerDragged(FXObject* o,FXSelector s,void* ptr)
{
	FXList::onVScrollerDragged(o, s, ptr);

	if ( current != -1 )
	{
		((CPropertyItem*)items[current])->setChildWidget(current * itemHeight + pos_y);
	}

	return 1;
}

/*
 * A new property has been selected. If a child widget is active
 * then remove it. Then add a new child widget for the newly
 * selected property.
 */
long CPropertyList::onLeftBtnPress(FXObject* o,FXSelector s,void* ptr)
{
	FXEvent* event = (FXEvent*)ptr;
	FXint index = getItemAt(event->win_x,event->win_y);

	FXint n = index * itemHeight + pos_y;

	if ( index != -1 )
	{
		//((CPropertyItem*)items[index])->setChildWidget(n);

		// From OnChanged
		if ( m_pCurrentItem /*(index == -1 || index != current) && current != -1*/ )
		{
			m_pCurrentItem->closeChildWidget();
			m_pCurrentItem = 0;
			target = NULL;
		}

		if ( index != -1 /*&& index != current*/ )
		{
			/*CPropertyItem**/ m_pCurrentItem = (CPropertyItem*)retrieveItem(index);
			/*target =*/ m_pCurrentItem->invokeChildWidget();
			m_pCurrentItem->refreshDescription();
		}
		// End From OnChanged
	}

	return FXList::onLeftBtnPress(o, s, ptr);
}

long CPropertyList::onLeftBtnRelease(FXObject* o,FXSelector s,void* ptr)
{
	return FXList::onLeftBtnRelease(o, s, ptr);
}

long CPropertyList::onKeyPress(FXObject*,FXSelector,void* ptr){
  FXEvent* event=(FXEvent*)ptr;
  FXint index;
  flags&=~FLAG_TIP;
  if(!isEnabled()) return 0;
  switch(event->code){
    case KEY_Page_Up:
      setPosition(pos_x,pos_y+verticalScrollbar()->getPage());
      return 1;
    case KEY_Page_Down:
      setPosition(pos_x,pos_y-verticalScrollbar()->getPage());
      return 1;
    case KEY_Up:
    case KEY_Down:
    case KEY_Home:
    case KEY_End:
      index=current;
      switch(event->code){
        case KEY_Up:
          index--;
          break;
        case KEY_Down: 
          index++;
          break;
        case KEY_Home: 
          index=0; 
          break;
        case KEY_End:  
          index=nitems-1; 
          break;
        }
      if(index<0) index=0;
      if(index>=nitems) index=nitems-1;
      handle(this,MKUINT(0,SEL_CHANGED),(void*)index);
      handle(this,MKUINT(0,SEL_ACTIVATE),ptr);                        // <== Added CMW
      if(0<=index && index!=current){
        if(event->state&(SHIFTMASK|CONTROLMASK)){
          if((options&SELECT_MASK)==LIST_EXTENDEDSELECT){
            extendSelection(index);
            }
          }
        }
      if((options&SELECT_MASK)==LIST_BROWSESELECT){
        handle(this,MKUINT(0,SEL_ACTIVATE),ptr);
        }
      flags&=~FLAG_UPDATE;
      return 1;
    case KEY_space:
    case KEY_KP_Space:
      handle(this,MKUINT(0,SEL_ACTIVATE),ptr);
      flags&=~FLAG_UPDATE;
	  break;
//      return 1;
    case KEY_Control_L:
    case KEY_Control_R:
    case KEY_Shift_L:
    case KEY_Shift_R:
    case KEY_Alt_L:
    case KEY_Alt_R:
      if(flags&FLAG_DODRAG){
        handle(this,MKUINT(0,SEL_DRAGGED),ptr);
        }
      return 1;
    }

	if ( current != -1 )
	{
		killFocus();

		FXWindow* pTarget = ((CPropertyItem*)items[current])->getChildWidget();

		if ( pTarget )
		{
			pTarget->setFocus();
			pTarget->handle(this,MKUINT(message,SEL_KEYPRESS),ptr);
		}
		return 1;
	}

  return 0;
  }

long CPropertyList::onKeyRelease(FXObject*,FXSelector,void* ptr){
  FXEvent* event=(FXEvent*)ptr;
  if(!isEnabled()) return 0;
  if(target && target->handle(this,MKUINT(message,SEL_KEYRELEASE),ptr)) return 1;
  switch(event->code){
    case KEY_Page_Up:
    case KEY_Page_Down:
      return 1;
    case KEY_Up:
    case KEY_Down:
    case KEY_Home:
    case KEY_End:
      flags|=FLAG_UPDATE;
      if((options&SELECT_MASK)==LIST_BROWSESELECT){
        handle(this,MKUINT(0,SEL_DEACTIVATE),ptr);
        }
      return 1;
    case KEY_space:
    case KEY_KP_Space:
      flags|=FLAG_UPDATE;
      handle(this,MKUINT(0,SEL_DEACTIVATE),ptr);
      return 1;
    case KEY_Shift_L:
    case KEY_Shift_R:
    case KEY_Control_L:
    case KEY_Control_R:
    case KEY_Alt_L:
    case KEY_Alt_R:
      if(flags&FLAG_DODRAG){
        handle(this,MKUINT(0,SEL_DRAGGED),ptr);
        }
      return 1;
    }
  return 0;
  }


long CPropertyList::onFocusIn(FXObject* sender,FXSelector sel,void* ptr)
{
  return 1;
}

long CPropertyList::onFocusOut(FXObject* sender,FXSelector sel,void* ptr)
{
  return 1;
}

long CPropertyList::onEnter(FXObject* sender,FXSelector sel,void* ptr)
{
	return 1;
}

long CPropertyList::onLeave(FXObject* sender,FXSelector sel,void* ptr)
{
	return 1;
}

long CPropertyList::onMotion(FXObject* o,FXSelector s,void* ptr)
{
	return 1;
}

long CPropertyList::onQueryTip(FXObject* sender,FXSelector,void*)
{
	if(0<=current)
	{
		CPropertyItem* p = static_cast<CPropertyItem*>(items[current]);
		FXString string=p->getTipText();
		sender->handle(this,MKUINT(FXWindow::ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&string);
		return 1;
	}
	return 0;
}

